<?php
require_once("base_connector.php");

//require_once("grid_dataprocessor.php");

/*! DataItem class for Grid component
**/

class GridDataItem extends DataItem{
	protected $row_attrs;//!< hash of row attributes
	protected $cell_attrs;//!< hash of cell attributes
	protected $userdata;
	
	function __construct($data,$name,$index=0){
		parent::__construct($data,$name,$index);
		
		$this->row_attrs=array();
		$this->cell_attrs=array();
		$this->userdata=array();
	}
	/*! set color of row
		
		@param color 
			color of row
	*/
	function set_row_color($color){
		$this->row_attrs["bgColor"]=$color;
	}
	/*! set style of row
		
		@param color 
			color of row
	*/
	function set_row_style($color){
		$this->row_attrs["style"]=$color;
	}
	/*! assign custom style to the cell
		
		@param name
			name of column
		@param value
			css style string
	*/
	function set_cell_style($name,$value){
		$this->set_cell_attribute($name,"style",$value);
	}
	/*! assign custom class to specific cell
		
		@param name
			name of column
		@param value
			css class name
	*/
	function set_cell_class($name,$value){
		$this->set_cell_attribute($name,"class",$value);
	}
	/*! set custom cell attribute
		
		@param name
			name of column
		@param attr
			name of attribute
		@param value
			value of attribute
	*/
	function set_cell_attribute($name,$attr,$value){
		if (!$this->cell_attrs[$name]) $this->cell_attrs[$name]=array();
		$this->cell_attrs[$name][$attr]=$value;
	}
	
	/*! set userdata section for the item
		
		@param name
			name of userdata
		@param value
			value of userdata
	*/
	function set_userdata($name, $value){
		$this->userdata[$name]=$value;
	}
		
	/*! set custom row attribute
		
		@param attr
			name of attribute
		@param value
			value of attribute
	*/
	function set_row_attribute($attr,$value){
		$this->row_attrs[$attr]=$value;
	}	
	
	/*! return self as XML string, starting part
	*/
	public function to_xml_start(){
		if ($this->skip) return "";
		
		$str="<row id='".$this->get_id()."'";
		foreach ($this->row_attrs as $k=>$v)
			$str.=" ".$k."='".$v."'";
		$str.=">";
		for ($i=0; $i < sizeof($this->config->data); $i++){ 
			$str.="<cell";
			$name=$this->config->data[$i]["name"];
			if (isset($this->cell_attrs[$name])){
				$cattrs=$this->cell_attrs[$name];
				foreach ($cattrs as $k => $v)
					$str.=" ".$k."='".$v."'";
			}
			$str.="><![CDATA[".$this->data[$name]."]]></cell>";
		}
		foreach ($this->userdata as $key => $value)
			$str.="<userdata name='".$key."'>".$value."</userdata>";
			
		return $str;
	}
	/*! return self as XML string, ending part
	*/
	public function to_xml_end(){
		if ($this->skip) return "";
		
		return "</row>";
	}
}
/*! Connector for the dhtmlxgrid
**/
class GridConnector extends Connector{
	protected $extra_output="";//!< extra info which need to be sent to client side
	private $options=array();//!< hash of OptionsConnector 
	
	/*! constructor
		
		Here initilization of all Masters occurs, execution timer initialized
		@param res 
			db connection resource
		@param type
			string , which hold type of database ( MySQL or Postgre ), optional, instead of short DB name, full name of DataWrapper-based class can be provided
		@param item_type
			name of class, which will be used for item rendering, optional, DataItem will be used by default
		@param data_type
			name of class which will be used for dataprocessor calls handling, optional, DataProcessor class will be used by default. 
	*/		
	public function __construct($res,$type=false,$item_type=false,$data_type=false){
		if (!$item_type) $item_type="GridDataItem";
		if (!$data_type) $data_type="GridDataProcessor";
		parent::__construct($res,$type,$item_type,$data_type);
	}


	protected function parse_request(){
		parent::parse_request();
		
		if (isset($_GET["dhx_colls"]))
			$this->fill_collections($_GET["dhx_colls"]);	
		
		if (isset($_GET["posStart"]) && isset($_GET["count"]))
			$this->request->set_limit($_GET["posStart"],$_GET["count"]);
	}
	protected function resolve_parameter($name){
		if (intval($name).""==$name)
			return $this->config->text[intval($name)]["db_name"];
		return $name;
	}
	
	/*! replace xml unsafe characters
		
		@param string 
			string to be escaped
		@return 
			escaped string
	*/	
	private function xmlentities($string) { 
   		return str_replace( array( '&', '"', "'", '<', '>', '’' ), array( '&amp;' , '&quot;', '&apos;' , '&lt;' , '&gt;', '&apos;' ), $string);
	}
		
	/*! assign options collection to the column
		
		@param name 
			name of the column
		@param options
			array or connector object
	*/
	public function set_options($name,$options){
		if (is_array($options)){
			$str="";
			foreach($options as $k => $v)
				$str.="<item value='".$this->xmlentities($k)."' label='".$this->xmlentities($v)."' />";
			$options=$str;
		}
		$this->options[$name]=$options;
	}
	/*! generates xml description for options collections
		
		@param list 
			comma separated list of column names, for which options need to be generated
	*/
	protected function fill_collections($list){
		$names=explode(",",$list);
		for ($i=0; $i < sizeof($names); $i++) { 
			$name = $this->resolve_parameter($names[$i]);
			if (!array_key_exists($name,$this->options)){
				$this->options[$name] = new DistinctOptionsConnector($this->get_connection(),$this->names["db_class"]);
				$c = new DataConfig($this->config);
				$r = new DataRequestConfig($this->request);
				$c->minimize($name);
				
				$this->options[$name]->render_connector($c,$r);
			} 
			
			$this->extra_output.="<coll_options for='{$names[$i]}'>";
			if (!is_string($this->options[$name]))
				$this->extra_output.=$this->options[$name]->render();
			else
				$this->extra_output.=$this->options[$name];
			$this->extra_output.="</coll_options>";
		}
	}
	
	/*! renders self as  xml, starting part
	*/
	protected function xml_start(){
		if ($this->dload){
			if ($pos=$this->request->get_start())
				return "<rows pos='".$pos."'>";
			else
				return "<rows total_count='".$this->sql->get_size($this->request)."'>";
		}
		else
			return "<rows>";
	}
	
	
	/*! renders self as  xml, ending part
	*/
	protected function xml_end(){
		return $this->extra_output."</rows>";
	}
	
	
	
	/*! attaching header functionality
	*/
	protected $headerDelimiter = ',';
	protected $headerNames = array();
	protected $headerWidths = array();
	protected $headerWidthsP = array();
	protected $headerAlign = array();
	protected $headerVAlign = array();
	protected $headerTypes = array();
	protected $headerSorts = array();
	protected $headerColors = array();
	protected $headerHidden = array();
	protected $headerIds = array();
	protected $headerAttaches = array();
	protected $footerAttaches = array();
	protected $headerWidthUnits = 'px';

	/*! sets delimiter for string arguments in attach header functions (default is ,)
		@param headerDelimiter
			string delimiter
	*/
	public function setHeaderDelimiter($headerDelimiter) {
		$this->headerDelimiter = $headerDelimiter;
	}

	/*! sets header
		@param names
		 array of names or string of names, delimited by headerDelimiter (default is ,)
	*/
	public function setHeader($names) {
		switch (gettype($names)) {
			case 'string':
				$this->headerNames = explode($this->headerDelimiter, $names);
				break;
			case 'array':
				foreach($names as $v) {
					$this->headerNames[] = $v;
				}
				break;
		}
		if ((!isset($_GET['editing']))&&(!isset($_GET['connector']))) {
			$this->event->attach("beforeOutput", Array($this, "attachHeaderToXML"));
		}
	}

	/*! sets init columns width in pixels
		@param wp
			array of widths or string of widths, delimited by headerDelimiter (default is ,)
	*/
	public function setInitWidths($wp) {
		switch (gettype($wp)) {
			case 'string':
				$this->headerWidths = explode($this->headerDelimiter, $wp);
				break;
			case 'array':
				foreach($wp as $v) {
					$this->headerWidths[] = $v;
				}
				break;
		}
		$this->headerWidthsUnits = 'px';
	}

	/*! sets init columns width in persents
		@param wp
			array of widths or string of widths, delimited by headerDelimiter (default is ,)
	*/
	public function setInitWidthsP($wp) {
		$this->setInitWidths($wp);
		$this->headerWidthsUnits = '%';
	}

	/*! sets columns align
		@param alStr
			array of aligns or string of aligns, delimited by headerDelimiter (default is ,)
	*/
	public function setColAlign($alStr) {
		$default = array("right", "left", "center", "justify");
		switch(gettype($alStr)) {
			case 'string':
				$this->headerAlign = explode($this->headerDelimiter, $alStr);
				for ($i = 0; $i < count($this->headerAlign); $i++) {
					if (!in_array($this->headerAlign[$i], $default)) {
						$this->headerAlign[$i] = 'left';
					}
				}
				break;
			case 'array':
				foreach($alStr as $v) {
					if (in_array($v, $default)) {
						$this->headerAlign[] = $v;
					} else {
						$this->headerAlign[] = 'left';
					}
				}
			break;
		}
	}

	/*! sets columns vertical align
		@param alStr
			array of vertical aligns or string of vertical aligns, delimited by headerDelimiter (default is ,)
	*/
	public function setColVAlign($alStr) {
		$default = array("baseline", "sub", "super", "top", "text-top", "middle", "bottom", "text-bottom");
		switch(gettype($alStr)) {
			case 'string':
				$this->headerVAlign = explode($this->headerDelimiter, $alStr);
				for ($i = 0; $i < count($this->headerVAlign); $i++) {
					if (!in_array($this->headerVAlign[$i], $default)) {
						$this->headerVAlign[$i] = 'top';
					}
				}
				break;
			case 'array':
				foreach($alStr as $v) {
					if (in_array($v, $default)) {
						$this->headerVAlign[] = $v;
					} else {
						$this->headerVAlign[] = 'top';
					}
				}
			break;
		}
	}

	/*! sets column types
		@param typeStr
			array of types or string of types, delimited by headerDelimiter (default is ,)
	*/
	public function setColTypes($typeStr) {
		switch(gettype($typeStr)) {
			case 'string':
				$this->headerTypes = explode($this->headerDelimiter, $typeStr);
				break;
			case 'array':
				foreach($typeStr as $v) {
					$this->headerTypes[] = $v;
				}
			break;
		}
	}

	/*! sets columns sorting
		@param sortStr
			array if sortings or string of sortings, delimited by headerDelimiter (default is ,)
	*/
	public function setColSorting($sortStr) {
		switch(gettype($sortStr)) {
			case 'string':
				$this->headerSorts = explode($this->headerDelimiter, $sortStr);
				break;
			case 'array':
				foreach($sortStr as $v) {
					$this->headerSorts[] = $v;
				}
			break;
		}
	}

	/*! sets columns colors
		@param colorStr
			array of colors or string of colors, delimited by headerDelimiter (default is ,)
			if (color should not be applied it's value should be null)
	*/
	public function setColColor($colorStr) {
		switch(gettype($colorStr)) {
			case 'string':
				$this->headerColors = explode($this->headerDelimiter, $colorStr);
				break;
			case 'array':
				foreach($colorStr as $v) {
					$this->headerColors[] = $v;
				}
			break;
		}
	}

	/*! sets hidden columns
		@param hidStr
			array of bool values or string of bool values, delimited by headerDelimiter (default is ,)
	*/
	public function setColHidden($hidStr) {
		switch(gettype($hidStr)) {
			case 'string':
				$this->headerHidden = explode($this->headerDelimiter, $hidStr);
				break;
			case 'array':
				foreach($hidStr as $v) {
					$this->headerHidden[] = $v;
				}
			break;
		}
	}

	/*! sets columns id
		@param idsStr
			array of ids or string of ids, delimited by headerDelimiter (default is ,)
	*/
	public function setColIds($idsStr) {
		switch(gettype($idsStr)) {
			case 'string':
				$this->headerIds = explode($this->headerDelimiter, $idsStr);
				break;
			case 'array':
				foreach($idsStr as $v) {
					$this->headerIds[] = $v;
				}
			break;
		}
	}

	/*! attaches header
		@param values
			array of header names or string of header names, delimited by headerDelimiter (default is ,)
		@param styles
			array of header styles or string of header styles, delimited by headerDelimiter (default is ,)
	*/
	public function attachHeader($values, $styles = null) {
		$header = Array();
		switch (gettype($values)) {
			case 'string':
				$header['values'] = $values;
				break;
			case 'array':
				$header['values'] = implode($this->headerDelimiter, $values);
				break;
		}
		if (styles != null) {
			switch (gettype($styles)) {
				case 'string':
					$header['styles'] = $styles;
					break;
				case 'array':
					$header['styles'] = implode($this->headerDelimiter, $styles);
					break;
			}
		} else {
			$header['styles'] = null;
		}
		$this->headerAttaches[] = $header;
	}

	/*! attaches footer
		@param values
			array of footer names or string of footer names, delimited by headerDelimiter (default is ,)
		@param styles
			array of footer styles or string of footer styles, delimited by headerDelimiter (default is ,)
	*/
	public function attachFooter($values, $styles = null) {
		$footer = Array();
		switch (gettype($values)) {
			case 'string':
				$footer['values'] = $values;
				break;
			case 'array':
				$footer['values'] = implode($this->headerDelimiter, $values);
				break;
		}
		if (styles != null) {
			switch (gettype($styles)) {
				case 'string':
					$footer['styles'] = $styles;
					break;
				case 'array':
					$footer['styles'] = implode($this->headerDelimiter, $styles);
					break;
			}
		} else {
			$footer['styles'] = null;
		}
		$this->footerAttaches[] = $footer;
	}

	/*! adds header configuration in output XML
	*/
	public function attachHeaderToXML() {
		$str = '<head>';
		$fillList = '';
		for ($i = 0; $i < count($this->headerNames); $i++) {
			$str .= '<column';
			$str .= (isset($this->headerIds[$i])) ? ' id="'.$this->headerIds[$i].'"' : '';
			$str .= (isset($this->headerTypes[$i])) ? ' type="'.$this->headerTypes[$i].'"' : ' type="ro"';
			$str .= (isset($this->headerWidths[$i])) ? ' width="'.$this->headerWidths[$i].'"' : '';
			$str .= (isset($this->headerAlign[$i])) ? ' align="'.$this->headerAlign[$i].'"' : '';
			$str .= (isset($this->headerVAlign[$i])) ? ' valign="'.$this->headerVAlign[$i].'"' : '';
			$str .= (isset($this->headerSorts[$i])) ? ' sort="'.$this->headerSorts[$i].'"' : '';
			$str .= ((isset($this->headerColors[$i]))&&($this->headerColors[$i] != 'null')) ? ' color="'.$this->headerColors[$i].'"' : '';
			$str .= (isset($this->headerHidden[$i])) ? ' hidden="'.$this->headerHidden[$i].'"' : '';
			$str .= '>'.$this->headerNames[$i].'</column>';
			
			if (isset($this->options[strtolower($this->headerNames[$i])])) {
				$fillList .= $i.',';
			}
		}
		$str .= '<settings><colwidth>'.$this->headerWidthsUnits.'</colwidth></settings>';
		if ((count($this->headerAttaches) > 0)||(count($this->footerAttaches) > 0)) {
			$str .= '<afterInit>';
		}
		for ($i = 0; $i < count($this->headerAttaches); $i++) {
			$str .= '<call command="attachHeader">';
			$str .= '<param>'.$this->headerAttaches[$i]['values'].'</param>';
			if ($this->headerAttaches[$i]['styles'] != null) {
				$str .= '<param>'.$this->headerAttaches[$i]['styles'].'</param>';
			}
			$str .= '</call>';
		}
		for ($i = 0; $i < count($this->footerAttaches); $i++) {
			$str .= '<call command="attachFooter">';
			$str .= '<param>'.$this->footerAttaches[$i]['values'].'</param>';
			if ($this->footerAttaches[$i]['styles'] != null) {
				$str .= '<param>'.$this->footerAttaches[$i]['styles'].'</param>';
			}
			$str .= '</call>';
		}
		if ((count($this->headerAttaches) > 0)||(count($this->footerAttaches) > 0)) {
			$str .= '</afterInit>';
		}

		$str .= '</head>';

		$fillList = substr($fillList, 0, strlen($fillList) - 1);
		if (strlen($fillList) > 0) {
			$this->fill_collections($fillList);
		}
		echo $str;
	}
}

/*! DataProcessor class for Grid component
**/
class GridDataProcessor extends DataProcessor{
	
	/*! convert incoming data name to valid db name
		converts c0..cN to valid field names
		@param data 
			data name from incoming request
		@return 
			related db_name
	*/
	function name_data($data){
		if ($data == "gr_id") return $this->config->id["name"];
		$parts=explode("c",$data);
		if ($parts[0]=="" && intval($parts[1])==$parts[1])
			return $this->config->text[intval($parts[1])]["name"];
		return $data;
	}
}

?>